@@ -18,13 +18,12 @@
  * <http://www.gnu.org/licenses/>.
  */
 
-#ifndef LIBVIRT_BHYVE_DOMAIN_H
-# define LIBVIRT_BHYVE_DOMAIN_H
+#pragma once
 
-# include "domain_addr.h"
-# include "domain_conf.h"
+#include "domain_addr.h"
+#include "domain_conf.h"
 
-# include "bhyve_monitor.h"
+#include "bhyve_monitor.h"
 
 typedef struct _bhyveDomainObjPrivate bhyveDomainObjPrivate;
 typedef bhyveDomainObjPrivate *bhyveDomainObjPrivatePtr;
@@ -42,5 +41,3 @@ extern virDomainDefParserConfig virBhyveDriverDomainDefParserConfig;
 extern virDomainXMLNamespace virBhyveDriverDomainXMLNamespace;
 
 bool bhyveDomainDefNeedsISAController(virDomainDefPtr def);
-
-#endif /* LIBVIRT_BHYVE_DOMAIN_H */
