@@ -43,6 +43,7 @@
 #include "virthread.h"
 #include "virlog.h"
 #include "virfile.h"
+#include "virpidfile.h"
 #include "virtypedparam.h"
 #include "virrandom.h"
 #include "virstring.h"
@@ -577,17 +578,18 @@ bhyveDomainDefineXML(virConnectPtr conn, const char *xml)
 }
 
 static int
-bhyveDomainUndefine(virDomainPtr domain)
+bhyveDomainUndefineFlags(virDomainPtr domain, unsigned int flags)
 {
     bhyveConnPtr privconn = domain->conn->privateData;
     virObjectEventPtr event = NULL;
     virDomainObjPtr vm;
     int ret = -1;
 
+    virCheckFlags(0, -1);
     if (!(vm = bhyveDomObjFromDomain(domain)))
         goto cleanup;
 
-    if (virDomainUndefineEnsureACL(domain->conn, vm->def) < 0)
+    if (virDomainUndefineFlagsEnsureACL(domain->conn, vm->def) < 0)
         goto cleanup;
 
     if (!vm->persistent) {
@@ -618,6 +620,12 @@ bhyveDomainUndefine(virDomainPtr domain)
     return ret;
 }
 
+static int
+bhyveDomainUndefine(virDomainPtr domain)
+{
+    return bhyveDomainUndefineFlags(domain, 0);
+}
+
 static int
 bhyveConnectListDomains(virConnectPtr conn, int *ids, int maxids)
 {
@@ -971,17 +979,19 @@ bhyveDomainCreateXML(virConnectPtr conn,
 }
 
 static int
-bhyveDomainDestroy(virDomainPtr dom)
+bhyveDomainDestroyFlags(virDomainPtr dom, unsigned int flags)
 {
     bhyveConnPtr privconn = dom->conn->privateData;
     virDomainObjPtr vm;
     virObjectEventPtr event = NULL;
     int ret = -1;
 
+    virCheckFlags(0, -1);
+
     if (!(vm = bhyveDomObjFromDomain(dom)))
         goto cleanup;
 
-    if (virDomainDestroyEnsureACL(dom->conn, vm->def) < 0)
+    if (virDomainDestroyFlagsEnsureACL(dom->conn, vm->def) < 0)
         goto cleanup;
 
     if (virDomainObjCheckActive(vm) < 0)
@@ -1002,15 +1012,23 @@ bhyveDomainDestroy(virDomainPtr dom)
 }
 
 static int
-bhyveDomainShutdown(virDomainPtr dom)
+bhyveDomainDestroy(virDomainPtr dom)
+{
+    return bhyveDomainDestroyFlags(dom, 0);
+}
+
+static int
+bhyveDomainShutdownFlags(virDomainPtr dom, unsigned int flags)
 {
     virDomainObjPtr vm;
     int ret = -1;
 
+    virCheckFlags(0, -1);
+
     if (!(vm = bhyveDomObjFromDomain(dom)))
         goto cleanup;
 
-    if (virDomainShutdownEnsureACL(dom->conn, vm->def) < 0)
+    if (virDomainShutdownFlagsEnsureACL(dom->conn, vm->def, flags) < 0)
         goto cleanup;
 
     if (virDomainObjCheckActive(vm) < 0)
@@ -1023,6 +1041,12 @@ bhyveDomainShutdown(virDomainPtr dom)
     return ret;
 }
 
+static int
+bhyveDomainShutdown(virDomainPtr dom)
+{
+    return bhyveDomainShutdownFlags(dom, 0);
+}
+
 static int
 bhyveDomainOpenConsole(virDomainPtr dom,
                        const char *dev_name ATTRIBUTE_UNUSED,
@@ -1180,6 +1204,9 @@ bhyveStateCleanup(void)
     virObjectUnref(bhyve_driver->config);
     virPortAllocatorRangeFree(bhyve_driver->remotePorts);
 
+    if (bhyve_driver->lockFD != -1)
+        virPidFileRelease(BHYVE_STATE_DIR, "driver", bhyve_driver->lockFD);
+
     virMutexDestroy(&bhyve_driver->lock);
     VIR_FREE(bhyve_driver);
 
@@ -1199,6 +1226,7 @@ bhyveStateInitialize(bool privileged,
     if (VIR_ALLOC(bhyve_driver) < 0)
         return -1;
 
+    bhyve_driver->lockFD = -1;
     if (virMutexInit(&bhyve_driver->lock) < 0) {
         VIR_FREE(bhyve_driver);
         return -1;
@@ -1251,6 +1279,10 @@ bhyveStateInitialize(bool privileged,
         goto cleanup;
     }
 
+    if ((bhyve_driver->lockFD =
+         virPidFileAcquire(BHYVE_STATE_DIR, "driver", false, getpid())) < 0)
+        goto cleanup;
+
     if (virDomainObjListLoadAllConfigs(bhyve_driver->domains,
                                        BHYVE_STATE_DIR,
                                        NULL, true,
@@ -1658,13 +1690,16 @@ static virHypervisorDriver bhyveHypervisorDriver = {
     .domainCreateWithFlags = bhyveDomainCreateWithFlags, /* 1.2.3 */
     .domainCreateXML = bhyveDomainCreateXML, /* 1.2.4 */
     .domainDestroy = bhyveDomainDestroy, /* 1.2.2 */
+    .domainDestroyFlags = bhyveDomainDestroyFlags, /* 5.6.0 */
     .domainShutdown = bhyveDomainShutdown, /* 1.3.3 */
+    .domainShutdownFlags = bhyveDomainShutdownFlags, /* 5.6.0 */
     .domainLookupByUUID = bhyveDomainLookupByUUID, /* 1.2.2 */
     .domainLookupByName = bhyveDomainLookupByName, /* 1.2.2 */
     .domainLookupByID = bhyveDomainLookupByID, /* 1.2.3 */
     .domainDefineXML = bhyveDomainDefineXML, /* 1.2.2 */
     .domainDefineXMLFlags = bhyveDomainDefineXMLFlags, /* 1.2.12 */
     .domainUndefine = bhyveDomainUndefine, /* 1.2.2 */
+    .domainUndefineFlags = bhyveDomainUndefineFlags, /* 5.6.0 */
     .domainGetOSType = bhyveDomainGetOSType, /* 1.2.21 */
     .domainGetXMLDesc = bhyveDomainGetXMLDesc, /* 1.2.2 */
     .domainIsActive = bhyveDomainIsActive, /* 1.2.2 */
