@@ -41,6 +41,7 @@
 #include "virstring.h"
 #include "virtypedparam.h"
 #include "viruuid.h"
+#include "virenum.h"
 
 #define VIR_FROM_THIS VIR_FROM_CAPABILITIES
 
@@ -49,7 +50,8 @@
 VIR_LOG_INIT("conf.capabilities");
 
 VIR_ENUM_DECL(virCapsHostPMTarget);
-VIR_ENUM_IMPL(virCapsHostPMTarget, VIR_NODE_SUSPEND_TARGET_LAST,
+VIR_ENUM_IMPL(virCapsHostPMTarget,
+              VIR_NODE_SUSPEND_TARGET_LAST,
               "suspend_mem", "suspend_disk", "suspend_hybrid",
 );
 
