@@ -484,7 +484,8 @@ VIR_ENUM_DECL(virStoragePartedFs);
                  VIR_CONNECT_LIST_STORAGE_POOLS_SHEEPDOG | \
                  VIR_CONNECT_LIST_STORAGE_POOLS_GLUSTER  | \
                  VIR_CONNECT_LIST_STORAGE_POOLS_ZFS      | \
-                 VIR_CONNECT_LIST_STORAGE_POOLS_VSTORAGE)
+                 VIR_CONNECT_LIST_STORAGE_POOLS_VSTORAGE | \
+                 VIR_CONNECT_LIST_STORAGE_POOLS_ISCSI_DIRECT)
 
 #define VIR_CONNECT_LIST_STORAGE_POOLS_FILTERS_ALL \
                 (VIR_CONNECT_LIST_STORAGE_POOLS_FILTERS_ACTIVE     | \
