@@ -25,12 +25,14 @@
 #include "internal.h"
 #include "datatypes.h"
 #include "virdomainobjlist.h"
+#include "checkpoint_conf.h"
 #include "snapshot_conf.h"
 #include "viralloc.h"
 #include "virfile.h"
 #include "virlog.h"
 #include "virstring.h"
 #include "virdomainsnapshotobjlist.h"
+#include "virdomaincheckpointobjlist.h"
 
 #define VIR_FROM_THIS VIR_FROM_DOMAIN
 
@@ -887,6 +889,15 @@ virDomainObjMatchFilter(virDomainObjPtr vm,
             return false;
     }
 
+    /* filter by checkpoint existence */
+    if (MATCH(VIR_CONNECT_LIST_DOMAINS_FILTERS_CHECKPOINT)) {
+        int nchk = virDomainListCheckpoints(vm->checkpoints, NULL, NULL,
+                                            NULL, 0);
+        if (!((MATCH(VIR_CONNECT_LIST_DOMAINS_HAS_CHECKPOINT) && nchk > 0) ||
+              (MATCH(VIR_CONNECT_LIST_DOMAINS_NO_CHECKPOINT) && nchk <= 0)))
+            return false;
+    }
+
     return true;
 }
 #undef MATCH
