@@ -58,6 +58,7 @@ int virNetSocketNewListenUNIX(const char *path,
                               virNetSocketPtr *addr);
 
 int virNetSocketNewListenFD(int fd,
+                            bool unlinkUNIX,
                             virNetSocketPtr *addr);
 
 int virNetSocketNewConnectTCP(const char *nodename,
@@ -127,6 +128,7 @@ bool virNetSocketIsLocal(virNetSocketPtr sock);
 
 bool virNetSocketHasPassFD(virNetSocketPtr sock);
 
+char *virNetSocketGetPath(virNetSocketPtr sock);
 int virNetSocketGetPort(virNetSocketPtr sock);
 
 int virNetSocketGetUNIXIdentity(virNetSocketPtr sock,
