@@ -620,7 +620,7 @@ virStorageBackendRBDGetVolNames(virStorageBackendRBDStatePtr ptr)
     size_t i;
 
     while (true) {
-        if (VIR_ALLOC_N(images, nimages) < 0)
+        if (VIR_REALLOC_N(images, nimages) < 0)
             goto error;
 
         rc = rbd_list2(ptr->ioctx, images, &nimages);
@@ -637,7 +637,7 @@ virStorageBackendRBDGetVolNames(virStorageBackendRBDStatePtr ptr)
     nnames = nimages;
 
     for (i = 0; i < nimages; i++)
-        VIR_STEAL_PTR(names[i], images->name);
+        VIR_STEAL_PTR(names[i], images[i].name);
 
     return names;
 
