@@ -67,7 +67,7 @@ static int virIdentityOnceInit(void)
     return 0;
 }
 
-VIR_ONCE_GLOBAL_INIT(virIdentity)
+VIR_ONCE_GLOBAL_INIT(virIdentity);
 
 /**
  * virIdentityGetCurrent:
