@@ -17,16 +17,15 @@
  * <http://www.gnu.org/licenses/>.
  */
 
-#ifndef LIBVIRT_VIRNETDEVMACVLAN_H
-# define LIBVIRT_VIRNETDEVMACVLAN_H
+#pragma once
 
-# include "internal.h"
-# include "virmacaddr.h"
-# include "virsocketaddr.h"
-# include "virnetdevbandwidth.h"
-# include "virnetdevvportprofile.h"
-# include "virnetdevvlan.h"
-# include "virenum.h"
+#include "internal.h"
+#include "virmacaddr.h"
+#include "virsocketaddr.h"
+#include "virnetdevbandwidth.h"
+#include "virnetdevvportprofile.h"
+#include "virnetdevvlan.h"
+#include "virenum.h"
 
 /* the mode type for macvtap devices */
 typedef enum {
@@ -52,8 +51,8 @@ typedef enum {
 /* libvirt will start macvtap/macvlan interface names with one of
  * these prefixes when it auto-generates the name
  */
-# define VIR_NET_GENERATED_MACVTAP_PREFIX "macvtap"
-# define VIR_NET_GENERATED_MACVLAN_PREFIX "macvlan"
+#define VIR_NET_GENERATED_MACVTAP_PREFIX "macvtap"
+#define VIR_NET_GENERATED_MACVLAN_PREFIX "macvlan"
 
 int virNetDevMacVLanReserveName(const char *name, bool quietfail);
 int virNetDevMacVLanReleaseName(const char *name);
@@ -112,4 +111,3 @@ int virNetDevMacVLanVPortProfileRegisterCallback(const char *ifname,
                                                  virNetDevVPortProfileOp vmOp)
 ATTRIBUTE_NONNULL(1) ATTRIBUTE_NONNULL(2) ATTRIBUTE_NONNULL(3)
 ATTRIBUTE_NONNULL(4) ATTRIBUTE_RETURN_CHECK;
-#endif /* LIBVIRT_VIRNETDEVMACVLAN_H */
