@@ -16,10 +16,9 @@
  * <http://www.gnu.org/licenses/>.
  */
 
-#ifndef LIBVIRT_VIRRANDOM_H
-# define LIBVIRT_VIRRANDOM_H
+#pragma once
 
-# include "internal.h"
+#include "internal.h"
 
 uint64_t virRandomBits(int nbits) ATTRIBUTE_NOINLINE;
 double virRandom(void);
@@ -27,5 +26,3 @@ uint32_t virRandomInt(uint32_t max);
 int virRandomBytes(unsigned char *buf, size_t buflen)
     ATTRIBUTE_NONNULL(1) ATTRIBUTE_RETURN_CHECK ATTRIBUTE_NOINLINE;
 int virRandomGenerateWWN(char **wwn, const char *virt_type) ATTRIBUTE_NOINLINE;
-
-#endif /* LIBVIRT_VIRRANDOM_H */
