@@ -1738,7 +1738,7 @@ const char *virGetEnvBlockSUID(const char *name)
 
 
 /**
- * virGetEnvBlockSUID:
+ * virGetEnvAllowSUID:
  * @name: the environment variable name
  *
  * Obtain an environment variable which is safe to
@@ -1784,122 +1784,6 @@ void virUpdateSelfLastChanged(const char *path)
     }
 }
 
-#ifndef WIN32
-
-/**
- * virGetListenFDs:
- *
- * Parse LISTEN_PID and LISTEN_FDS passed from caller.
- *
- * Returns number of passed FDs.
- */
-unsigned int
-virGetListenFDs(void)
-{
-    const char *pidstr;
-    const char *fdstr;
-    size_t i = 0;
-    unsigned long long procid;
-    unsigned int nfds;
-
-    VIR_DEBUG("Setting up networking from caller");
-
-    if (!(pidstr = virGetEnvAllowSUID("LISTEN_PID"))) {
-        VIR_DEBUG("No LISTEN_PID from caller");
-        return 0;
-    }
-
-    if (virStrToLong_ull(pidstr, NULL, 10, &procid) < 0) {
-        VIR_DEBUG("Malformed LISTEN_PID from caller %s", pidstr);
-        return 0;
-    }
-
-    if ((pid_t)procid != getpid()) {
-        VIR_DEBUG("LISTEN_PID %s is not for us %lld",
-                  pidstr, (long long) getpid());
-        return 0;
-    }
-
-    if (!(fdstr = virGetEnvAllowSUID("LISTEN_FDS"))) {
-        VIR_DEBUG("No LISTEN_FDS from caller");
-        return 0;
-    }
-
-    if (virStrToLong_ui(fdstr, NULL, 10, &nfds) < 0) {
-        VIR_DEBUG("Malformed LISTEN_FDS from caller %s", fdstr);
-        return 0;
-    }
-
-    unsetenv("LISTEN_PID");
-    unsetenv("LISTEN_FDS");
-
-    VIR_DEBUG("Got %u file descriptors", nfds);
-
-    for (i = 0; i < nfds; i++) {
-        int fd = STDERR_FILENO + i + 1;
-
-        VIR_DEBUG("Disabling inheritance of passed FD %d", fd);
-
-        if (virSetInherit(fd, false) < 0)
-            VIR_WARN("Couldn't disable inheritance of passed FD %d", fd);
-    }
-
-    return nfds;
-}
-
-#else /* WIN32 */
-
-unsigned int
-virGetListenFDs(void)
-{
-    return 0;
-}
-
-#endif /* WIN32 */
-
-#ifdef HAVE_SYS_UN_H
-char *virGetUNIXSocketPath(int fd)
-{
-    union {
-        struct sockaddr sa;
-        struct sockaddr_storage ss;
-        struct sockaddr_un un;
-    } addr = { .ss = { 0 } };
-    socklen_t len = sizeof(addr.ss);
-    char *path;
-
-    if (getsockname(fd, &addr.sa, &len) < 0) {
-        virReportSystemError(errno, _("Unable to get address of FD %d"), fd);
-        return NULL;
-    }
-
-    if (addr.ss.ss_family != AF_UNIX) {
-        virReportSystemError(EINVAL, _("FD %d is not a UNIX socket, has af=%d"),
-                             fd, addr.ss.ss_family);
-        return NULL;
-    }
-
-    if (addr.un.sun_path[0] == '\0')
-        addr.un.sun_path[0] = '@';
-
-    if (VIR_ALLOC_N(path, sizeof(addr.un.sun_path) + 1) < 0)
-        return NULL;
-
-    memcpy(path, addr.un.sun_path, sizeof(addr.un.sun_path));
-    path[sizeof(addr.un.sun_path)] = '\0';
-    return path;
-}
-
-#else /* HAVE_SYS_UN_H */
-
-char *virGetUNIXSocketPath(int fd ATTRIBUTE_UNUSED)
-{
-    virReportSystemError(ENOSYS, "%s",
-                         _("UNIX sockets not supported on this platform"));
-    return NULL;
-}
-
-#endif /* HAVE_SYS_UN_H */
 
 #ifndef WIN32
 long virGetSystemPageSize(void)
