@@ -24,6 +24,8 @@
 
 #include <config.h>
 
+#include <regex.h>
+
 #include "internal.h"
 #include "virerror.h"
 #include "virconf.h"
@@ -761,6 +763,152 @@ xenParseVfb(virConfPtr conf, virDomainDefPtr def)
 }
 
 
+/**
+  * xenParseSxprChar:
+  * @value: A string describing a character device.
+  * @tty: the console pty path
+  *
+  * Parse the xend S-expression for description of a character device.
+  *
+  * Returns a character device object or NULL in case of failure.
+  */
+static virDomainChrDefPtr
+xenParseSxprChar(const char *value,
+                 const char *tty)
+{
+    const char *prefix;
+    char *tmp;
+    virDomainChrDefPtr def;
+
+    if (!(def = virDomainChrDefNew(NULL)))
+        return NULL;
+
+    prefix = value;
+
+    if (value[0] == '/') {
+        def->source->type = VIR_DOMAIN_CHR_TYPE_DEV;
+        if (VIR_STRDUP(def->source->data.file.path, value) < 0)
+            goto error;
+    } else {
+        if ((tmp = strchr(value, ':')) != NULL) {
+            *tmp = '\0';
+            value = tmp + 1;
+        }
+
+        if (STRPREFIX(prefix, "telnet")) {
+            def->source->type = VIR_DOMAIN_CHR_TYPE_TCP;
+            def->source->data.tcp.protocol = VIR_DOMAIN_CHR_TCP_PROTOCOL_TELNET;
+        } else {
+            if ((def->source->type = virDomainChrTypeFromString(prefix)) < 0) {
+                virReportError(VIR_ERR_INTERNAL_ERROR,
+                               _("unknown chr device type '%s'"), prefix);
+                goto error;
+            }
+        }
+    }
+
+    switch (def->source->type) {
+    case VIR_DOMAIN_CHR_TYPE_PTY:
+        if (VIR_STRDUP(def->source->data.file.path, tty) < 0)
+            goto error;
+        break;
+
+    case VIR_DOMAIN_CHR_TYPE_FILE:
+    case VIR_DOMAIN_CHR_TYPE_PIPE:
+        if (VIR_STRDUP(def->source->data.file.path, value) < 0)
+            goto error;
+        break;
+
+    case VIR_DOMAIN_CHR_TYPE_TCP:
+    {
+        const char *offset = strchr(value, ':');
+        const char *offset2;
+
+        if (offset == NULL) {
+            virReportError(VIR_ERR_INTERNAL_ERROR,
+                           "%s", _("malformed char device string"));
+            goto error;
+        }
+
+        if (offset != value &&
+            VIR_STRNDUP(def->source->data.tcp.host, value, offset - value) < 0)
+            goto error;
+
+        offset2 = strchr(offset, ',');
+        offset++;
+        if (VIR_STRNDUP(def->source->data.tcp.service, offset,
+                        offset2 ? offset2 - offset : -1) < 0)
+            goto error;
+
+        if (offset2 && strstr(offset2, ",server"))
+            def->source->data.tcp.listen = true;
+    }
+    break;
+
+    case VIR_DOMAIN_CHR_TYPE_UDP:
+    {
+        const char *offset = strchr(value, ':');
+        const char *offset2, *offset3;
+
+        if (offset == NULL) {
+            virReportError(VIR_ERR_INTERNAL_ERROR,
+                           "%s", _("malformed char device string"));
+            goto error;
+        }
+
+        if (offset != value &&
+            VIR_STRNDUP(def->source->data.udp.connectHost, value, offset - value) < 0)
+            goto error;
+
+        offset2 = strchr(offset, '@');
+        if (offset2 != NULL) {
+            if (VIR_STRNDUP(def->source->data.udp.connectService,
+                            offset + 1, offset2 - offset - 1) < 0)
+                goto error;
+
+            offset3 = strchr(offset2, ':');
+            if (offset3 == NULL) {
+                virReportError(VIR_ERR_INTERNAL_ERROR,
+                               "%s", _("malformed char device string"));
+                goto error;
+            }
+
+            if (offset3 > (offset2 + 1) &&
+                VIR_STRNDUP(def->source->data.udp.bindHost,
+                            offset2 + 1, offset3 - offset2 - 1) < 0)
+                goto error;
+
+            if (VIR_STRDUP(def->source->data.udp.bindService, offset3 + 1) < 0)
+                goto error;
+        } else {
+            if (VIR_STRDUP(def->source->data.udp.connectService, offset + 1) < 0)
+                goto error;
+        }
+    }
+    break;
+
+    case VIR_DOMAIN_CHR_TYPE_UNIX:
+    {
+        const char *offset = strchr(value, ',');
+        if (VIR_STRNDUP(def->source->data.nix.path, value,
+                        offset ? offset - value : -1) < 0)
+            goto error;
+
+        if (offset != NULL &&
+            strstr(offset, ",server") != NULL)
+            def->source->data.nix.listen = true;
+    }
+    break;
+    }
+
+    return def;
+
+ error:
+    virDomainChrDefFree(def);
+    return NULL;
+}
+
+
 static int
 xenParseCharDev(virConfPtr conf, virDomainDefPtr def, const char *nativeFormat)
 {
@@ -926,6 +1074,66 @@ xenParseVifBridge(virDomainNetDefPtr net, char *bridge)
 }
 
 
+static const char *vif_bytes_per_sec_re = "^[0-9]+[GMK]?[Bb]/s$";
+
+static int
+xenParseSxprVifRate(const char *rate, unsigned long long *kbytes_per_sec)
+{
+    char *trate = NULL;
+    char *p;
+    regex_t rec;
+    int err;
+    char *suffix;
+    unsigned long long tmp;
+    int ret = -1;
+
+    if (VIR_STRDUP(trate, rate) < 0)
+        return -1;
+
+    p = strchr(trate, '@');
+    if (p != NULL)
+        *p = 0;
+
+    err = regcomp(&rec, vif_bytes_per_sec_re, REG_EXTENDED|REG_NOSUB);
+    if (err != 0) {
+        char error[100];
+        regerror(err, &rec, error, sizeof(error));
+        virReportError(VIR_ERR_INTERNAL_ERROR,
+                       _("Failed to compile regular expression '%s': %s"),
+                       vif_bytes_per_sec_re, error);
+        goto cleanup;
+    }
+
+    if (regexec(&rec, trate, 0, NULL, 0)) {
+        virReportError(VIR_ERR_INTERNAL_ERROR,
+                       _("Invalid rate '%s' specified"), rate);
+        goto cleanup;
+    }
+
+    if (virStrToLong_ull(rate, &suffix, 10, &tmp)) {
+        virReportError(VIR_ERR_INTERNAL_ERROR,
+                       _("Failed to parse rate '%s'"), rate);
+        goto cleanup;
+    }
+
+    if (*suffix == 'G')
+       tmp *= 1024 * 1024;
+    else if (*suffix == 'M')
+       tmp *= 1024;
+
+    if (*suffix == 'b' || *(suffix + 1) == 'b')
+       tmp /= 8;
+
+    *kbytes_per_sec = tmp;
+    ret = 0;
+
+ cleanup:
+    regfree(&rec);
+    VIR_FREE(trate);
+    return ret;
+}
+
+
 static virDomainNetDefPtr
 xenParseVif(char *entry, const char *vif_typename)
 {
@@ -1137,6 +1345,87 @@ xenParseVifList(virConfPtr conf, virDomainDefPtr def, const char *vif_typename)
 }
 
 
+/**
+ * xenParseSxprSound:
+ * @def: the domain config
+ * @str: comma separated list of sound models
+ *
+ * This parses out sound devices from the domain S-expression
+ *
+ * Returns 0 if successful or -1 if failed.
+ */
+static int
+xenParseSxprSound(virDomainDefPtr def,
+                  const char *str)
+{
+    if (STREQ(str, "all")) {
+        size_t i;
+
+        /*
+         * Special compatibility code for Xen with a bogus
+         * sound=all in config.
+         *
+         * NB deliberately, don't include all possible
+         * sound models anymore, just the 2 that were
+         * historically present in Xen's QEMU.
+         *
+         * ie just es1370 + sb16.
+         *
+         * Hence use of MODEL_ES1370 + 1, instead of MODEL_LAST
+         */
+
+        if (VIR_ALLOC_N(def->sounds,
+                        VIR_DOMAIN_SOUND_MODEL_ES1370 + 1) < 0)
+            return -1;
+
+
+        for (i = 0; i < (VIR_DOMAIN_SOUND_MODEL_ES1370 + 1); i++) {
+            virDomainSoundDefPtr sound;
+            if (VIR_ALLOC(sound) < 0)
+                return -1;
+            sound->model = i;
+            def->sounds[def->nsounds++] = sound;
+        }
+    } else {
+        char model[10];
+        const char *offset = str, *offset2;
+
+        do {
+            int len;
+            virDomainSoundDefPtr sound;
+            offset2 = strchr(offset, ',');
+            if (offset2)
+                len = (offset2 - offset);
+            else
+                len = strlen(offset);
+            if (virStrncpy(model, offset, len, sizeof(model)) < 0) {
+                virReportError(VIR_ERR_INTERNAL_ERROR,
+                               _("Sound model %s too big for destination"),
+                               offset);
+                return -1;
+            }
+
+            if (VIR_ALLOC(sound) < 0)
+                return -1;
+
+            if ((sound->model = virDomainSoundModelTypeFromString(model)) < 0) {
+                VIR_FREE(sound);
+                return -1;
+            }
+
+            if (VIR_APPEND_ELEMENT(def->sounds, def->nsounds, sound) < 0) {
+                virDomainSoundDefFree(sound);
+                return -1;
+            }
+
+            offset = offset2 ? offset2 + 1 : NULL;
+        } while (offset);
+    }
+
+    return 0;
+}
+
+
 static int
 xenParseEmulatedDevices(virConfPtr conf, virDomainDefPtr def)
 {
@@ -1260,6 +1549,85 @@ xenParseConfigCommon(virConfPtr conf,
 }
 
 
+/**
+ * xenFormatSxprChr:
+ * @def: the domain config
+ * @buf: a buffer for the result S-expression
+ *
+ * Convert the character device part of the domain config into a S-expression
+ * in buf.
+ *
+ * Returns 0 in case of success, -1 in case of error
+ */
+static int
+xenFormatSxprChr(virDomainChrDefPtr def,
+                 virBufferPtr buf)
+{
+    const char *type = virDomainChrTypeToString(def->source->type);
+
+    if (!type) {
+        virReportError(VIR_ERR_INTERNAL_ERROR,
+                       "%s", _("unexpected chr device type"));
+        return -1;
+    }
+
+    switch (def->source->type) {
+    case VIR_DOMAIN_CHR_TYPE_NULL:
+    case VIR_DOMAIN_CHR_TYPE_STDIO:
+    case VIR_DOMAIN_CHR_TYPE_VC:
+    case VIR_DOMAIN_CHR_TYPE_PTY:
+        virBufferAdd(buf, type, -1);
+        break;
+
+    case VIR_DOMAIN_CHR_TYPE_FILE:
+    case VIR_DOMAIN_CHR_TYPE_PIPE:
+        virBufferAsprintf(buf, "%s:", type);
+        virBufferEscapeSexpr(buf, "%s", def->source->data.file.path);
+        break;
+
+    case VIR_DOMAIN_CHR_TYPE_DEV:
+        virBufferEscapeSexpr(buf, "%s", def->source->data.file.path);
+        break;
+
+    case VIR_DOMAIN_CHR_TYPE_TCP:
+        virBufferAsprintf(buf, "%s:%s:%s%s",
+                          (def->source->data.tcp.protocol
+                           == VIR_DOMAIN_CHR_TCP_PROTOCOL_RAW ?
+                           "tcp" : "telnet"),
+                          NULLSTR_EMPTY(def->source->data.tcp.host),
+                          NULLSTR_EMPTY(def->source->data.tcp.service),
+                          (def->source->data.tcp.listen ?
+                           ",server,nowait" : ""));
+        break;
+
+    case VIR_DOMAIN_CHR_TYPE_UDP:
+        virBufferAsprintf(buf, "%s:%s:%s@%s:%s", type,
+                          NULLSTR_EMPTY(def->source->data.udp.connectHost),
+                          NULLSTR_EMPTY(def->source->data.udp.connectService),
+                          NULLSTR_EMPTY(def->source->data.udp.bindHost),
+                          NULLSTR_EMPTY(def->source->data.udp.bindService));
+        break;
+
+    case VIR_DOMAIN_CHR_TYPE_UNIX:
+        virBufferAsprintf(buf, "%s:", type);
+        virBufferEscapeSexpr(buf, "%s", def->source->data.nix.path);
+        if (def->source->data.nix.listen)
+            virBufferAddLit(buf, ",server,nowait");
+        break;
+
+    default:
+        virReportError(VIR_ERR_CONFIG_UNSUPPORTED,
+                       _("unsupported chr device type '%s'"), type);
+        return -1;
+    }
+
+    if (virBufferCheckError(buf) < 0)
+        return -1;
+
+    return 0;
+}
+
+
 static int
 xenFormatSerial(virConfValuePtr list, virDomainChrDefPtr serial)
 {
@@ -2006,25 +2374,34 @@ xenFormatVfb(virConfPtr conf, virDomainDefPtr def)
 static int
 xenFormatSound(virConfPtr conf, virDomainDefPtr def)
 {
-    if (def->os.type == VIR_DOMAIN_OSTYPE_HVM) {
-        if (def->sounds) {
-            virBuffer buf = VIR_BUFFER_INITIALIZER;
-            char *str = NULL;
-            int ret = xenFormatSxprSound(def, &buf);
+    VIR_AUTOCLEAN(virBuffer) buf = VIR_BUFFER_INITIALIZER;
+    const char * model;
+    VIR_AUTOFREE(char *) str = NULL;
+    size_t i;
 
-            str = virBufferContentAndReset(&buf);
-            if (ret == 0)
-                ret = xenConfigSetString(conf, "soundhw", str);
+    if (def->os.type != VIR_DOMAIN_OSTYPE_HVM ||
+        !def->sounds)
+        return 0;
 
-            VIR_FREE(str);
-            if (ret < 0)
-                return -1;
+    for (i = 0; i < def->nsounds; i++) {
+        if (!(model = virDomainSoundModelTypeToString(def->sounds[i]->model))) {
+            virReportError(VIR_ERR_INTERNAL_ERROR,
+                           _("unexpected sound model %d"),
+                           def->sounds[i]->model);
+            return -1;
         }
+        if (i)
+            virBufferAddChar(&buf, ',');
+        virBufferEscapeSexpr(&buf, "%s", model);
     }
 
-    return 0;
-}
+    if (virBufferCheckError(&buf) < 0)
+        return -1;
 
+    str = virBufferContentAndReset(&buf);
+
+    return xenConfigSetString(conf, "soundhw", str);
+}
 
 
 static int
