@@ -1003,63 +1003,6 @@ test23(const void *unused ATTRIBUTE_UNUSED)
     return ret;
 }
 
-static int test24(const void *unused ATTRIBUTE_UNUSED)
-{
-    char *pidfile = virPidFileBuildPath(abs_builddir, "commandhelper");
-    char *prefix = NULL;
-    int newfd1 = dup(STDERR_FILENO);
-    int newfd2 = dup(STDERR_FILENO);
-    int newfd3 = dup(STDERR_FILENO);
-    int ret = -1;
-    pid_t pid;
-    virCommandPtr cmd = virCommandNew(abs_builddir "/commandhelper");
-
-    if (!pidfile)
-        goto cleanup;
-
-    if (VIR_CLOSE(newfd1) < 0)
-        printf("Cannot close fd %d\n", newfd1);
-
-    virCommandSetPidFile(cmd, pidfile);
-    virCommandDaemonize(cmd);
-    virCommandPassFD(cmd, newfd2, VIR_COMMAND_PASS_FD_CLOSE_PARENT);
-    virCommandPassFD(cmd, newfd3, VIR_COMMAND_PASS_FD_CLOSE_PARENT);
-    newfd2 = newfd3 = -1;
-    virCommandPassListenFDs(cmd);
-
-    if (virCommandRun(cmd, NULL) < 0) {
-        printf("Cannot run child %s\n", virGetLastErrorMessage());
-        goto cleanup;
-    }
-
-    if (virPidFileRead(abs_builddir, "commandhelper", &pid) < 0) {
-        printf("cannot read pidfile\n");
-        goto cleanup;
-    }
-
-    if (virAsprintf(&prefix,
-                    "ENV:LISTEN_FDS=2\nENV:LISTEN_PID=%u\n",
-                    pid) < 0)
-        goto cleanup;
-
-    while (kill(pid, 0) != -1)
-        usleep(100*1000);
-
-    ret = checkoutput("test24", prefix);
-
- cleanup:
-    if (pidfile)
-        unlink(pidfile);
-    VIR_FREE(pidfile);
-    VIR_FREE(prefix);
-    virCommandFree(cmd);
-    VIR_FORCE_CLOSE(newfd1);
-    VIR_FORCE_CLOSE(newfd2);
-    VIR_FORCE_CLOSE(newfd3);
-    return ret;
-}
-
-
 static int test25(const void *unused ATTRIBUTE_UNUSED)
 {
     int ret = -1;
@@ -1196,6 +1139,120 @@ static int test26(const void *unused ATTRIBUTE_UNUSED)
     return ret;
 }
 
+static int test27(const void *unused ATTRIBUTE_UNUSED)
+{
+    virCommandPtr cmd = virCommandNew(abs_builddir "/commandhelper");
+    int pipe1[2];
+    int pipe2[2];
+    int ret = -1;
+    size_t buflen = 1024 * 128;
+    char *buffer0 = NULL;
+    char *buffer1 = NULL;
+    char *buffer2 = NULL;
+    char *outactual = NULL;
+    char *erractual = NULL;
+    char *outexpect = NULL;
+# define TEST27_OUTEXPECT_TEMP "BEGIN STDOUT\n" \
+        "%s%s%s" \
+        "END STDOUT\n"
+    char *errexpect = NULL;
+# define TEST27_ERREXPECT_TEMP "BEGIN STDERR\n" \
+        "%s%s%s" \
+        "END STDERR\n"
+
+    if (VIR_ALLOC_N(buffer0, buflen) < 0 ||
+        VIR_ALLOC_N(buffer1, buflen) < 0 ||
+        VIR_ALLOC_N(buffer2, buflen) < 0)
+        goto cleanup;
+
+    memset(buffer0, 'H', buflen - 2);
+    buffer0[buflen - 2] = '\n';
+    buffer0[buflen - 1] = 0;
+
+    memset(buffer1, '1', buflen - 2);
+    buffer1[buflen - 2] = '\n';
+    buffer1[buflen - 1] = 0;
+
+    memset(buffer2, '2', buflen - 2);
+    buffer2[buflen - 2] = '\n';
+    buffer2[buflen - 1] = 0;
+
+    if (virAsprintf(&outexpect, TEST27_OUTEXPECT_TEMP,
+                    buffer0, buffer1, buffer2) < 0 ||
+        virAsprintf(&errexpect, TEST27_ERREXPECT_TEMP,
+                    buffer0, buffer1, buffer2) < 0) {
+        printf("Could not virAsprintf expected output\n");
+        goto cleanup;
+    }
+
+    if (pipe(pipe1) < 0 || pipe(pipe2) < 0) {
+        printf("Could not create pipe: %s\n", strerror(errno));
+        goto cleanup;
+    }
+
+    if (virCommandSetSendBuffer(cmd, pipe1[1],
+            (unsigned char *)buffer1, buflen - 1)  < 0 ||
+        virCommandSetSendBuffer(cmd, pipe2[1],
+            (unsigned char *)buffer2, buflen - 1) < 0) {
+        printf("Could not set send buffers\n");
+        goto cleanup;
+    }
+    pipe1[1] = 0;
+    pipe2[1] = 0;
+    buffer1 = NULL;
+    buffer2 = NULL;
+
+    virCommandAddArg(cmd, "--readfd");
+    virCommandAddArgFormat(cmd, "%d", pipe1[0]);
+    virCommandPassFD(cmd, pipe1[0], 0);
+
+    virCommandAddArg(cmd, "--readfd");
+    virCommandAddArgFormat(cmd, "%d", pipe2[0]);
+    virCommandPassFD(cmd, pipe2[0], 0);
+
+    virCommandSetInputBuffer(cmd, buffer0);
+    virCommandSetOutputBuffer(cmd, &outactual);
+    virCommandSetErrorBuffer(cmd, &erractual);
+
+    if (virCommandRun(cmd, NULL) < 0) {
+        printf("Cannot run child %s\n", virGetLastErrorMessage());
+        goto cleanup;
+    }
+
+    if (!outactual || !erractual)
+        goto cleanup;
+
+    if (STRNEQ(outactual, outexpect)) {
+        virTestDifference(stderr, outexpect, outactual);
+        goto cleanup;
+    }
+    if (STRNEQ(erractual, errexpect)) {
+        virTestDifference(stderr, errexpect, erractual);
+        goto cleanup;
+    }
+
+    if (checkoutput("test27", NULL) < 0)
+        goto cleanup;
+
+    ret = 0;
+
+ cleanup:
+    virCommandFree(cmd);
+    VIR_FORCE_CLOSE(pipe1[0]);
+    VIR_FORCE_CLOSE(pipe2[0]);
+    VIR_FORCE_CLOSE(pipe1[1]);
+    VIR_FORCE_CLOSE(pipe2[1]);
+    VIR_FREE(buffer0);
+    VIR_FREE(buffer1);
+    VIR_FREE(buffer2);
+    VIR_FREE(outactual);
+    VIR_FREE(erractual);
+    VIR_FREE(outexpect);
+    VIR_FREE(errexpect);
+
+    return ret;
+}
+
 static void virCommandThreadWorker(void *opaque)
 {
     virCommandTestDataPtr test = opaque;
@@ -1347,9 +1404,9 @@ mymain(void)
     DO_TEST(test21);
     DO_TEST(test22);
     DO_TEST(test23);
-    DO_TEST(test24);
     DO_TEST(test25);
     DO_TEST(test26);
+    DO_TEST(test27);
 
     virMutexLock(&test->lock);
     if (test->running) {
