@@ -30,6 +30,9 @@
 
 # include "qemu/qemu_command.h"
 
+# define LIBVIRT_SNAPSHOT_CONF_PRIV_H_ALLOW
+# include "conf/snapshot_conf_priv.h"
+
 # define VIR_FROM_THIS VIR_FROM_NONE
 
 VIR_LOG_INIT("tests.storagetest");
@@ -67,7 +70,8 @@ testBackingXMLjsonXML(const void *args)
         return -1;
     }
 
-    if (!(backendprops = qemuBlockStorageSourceGetBackendProps(xmlsrc, true))) {
+    if (!(backendprops = qemuBlockStorageSourceGetBackendProps(xmlsrc, true, false,
+                                                               false))) {
         fprintf(stderr, "failed to format disk source json\n");
         return -1;
     }
@@ -115,6 +119,9 @@ struct testQemuDiskXMLToJSONData {
     virJSONValuePtr *props;
     size_t nprops;
 
+    virJSONValuePtr *propssrc;
+    size_t npropssrc;
+
     virQEMUCapsPtr qemuCaps;
 };
 
@@ -127,8 +134,13 @@ testQemuDiskXMLToPropsClear(struct testQemuDiskXMLToJSONData *data)
     for (i = 0; i < data->nprops; i++)
         virJSONValueFree(data->props[i]);
 
+    for (i = 0; i < data->npropssrc; i++)
+        virJSONValueFree(data->propssrc[i]);
+
     data->nprops = 0;
     VIR_FREE(data->props);
+    data->npropssrc = 0;
+    VIR_FREE(data->propssrc);
 }
 
 
@@ -180,6 +192,7 @@ testQemuDiskXMLToProps(const void *opaque)
     virStorageSourcePtr n;
     virJSONValuePtr formatProps = NULL;
     virJSONValuePtr storageProps = NULL;
+    VIR_AUTOPTR(virJSONValue) storageSrcOnlyProps = NULL;
     char *xmlpath = NULL;
     char *xmlstr = NULL;
     int ret = -1;
@@ -213,7 +226,8 @@ testQemuDiskXMLToProps(const void *opaque)
             goto cleanup;
 
         if (!(formatProps = qemuBlockStorageSourceGetBlockdevProps(n)) ||
-            !(storageProps = qemuBlockStorageSourceGetBackendProps(n, false))) {
+            !(storageSrcOnlyProps = qemuBlockStorageSourceGetBackendProps(n, false, true, true)) ||
+            !(storageProps = qemuBlockStorageSourceGetBackendProps(n, false, false, true))) {
             if (!data->fail) {
                 VIR_TEST_VERBOSE("failed to generate qemu blockdev props\n");
                 goto cleanup;
@@ -224,7 +238,8 @@ testQemuDiskXMLToProps(const void *opaque)
         }
 
         if (VIR_APPEND_ELEMENT(data->props, data->nprops, formatProps) < 0 ||
-            VIR_APPEND_ELEMENT(data->props, data->nprops, storageProps) < 0)
+            VIR_APPEND_ELEMENT(data->props, data->nprops, storageProps) < 0 ||
+            VIR_APPEND_ELEMENT(data->propssrc, data->npropssrc, storageSrcOnlyProps) < 0)
             goto cleanup;
     }
 
@@ -268,6 +283,23 @@ testQemuDiskXMLToPropsValidateSchema(const void *opaque)
 
         virBufferFreeAndReset(&debug);
     }
+
+    for (i = 0; i < data->npropssrc; i++) {
+        if (testQEMUSchemaValidate(data->propssrc[i], data->schemaroot,
+                                   data->schema, &debug) < 0) {
+            debugmsg = virBufferContentAndReset(&debug);
+            propsstr = virJSONValueToString(data->propssrc[i], true);
+            VIR_TEST_VERBOSE("json does not conform to QAPI schema");
+            VIR_TEST_DEBUG("json:\n%s\ndoes not match schema. Debug output:\n %s",
+                           propsstr, NULLSTR(debugmsg));
+            VIR_FREE(debugmsg);
+            VIR_FREE(propsstr);
+            ret = -1;
+        }
+
+        virBufferFreeAndReset(&debug);
+    }
+
     return ret;
 }
 
@@ -313,6 +345,172 @@ testQemuDiskXMLToPropsValidateFile(const void *opaque)
 }
 
 
+struct testQemuImageCreateData {
+    const char *name;
+    const char *backingname;
+    virHashTablePtr schema;
+    virJSONValuePtr schemaroot;
+    virQEMUDriverPtr driver;
+    virQEMUCapsPtr qemuCaps;
+};
+
+static const char *testQemuImageCreatePath = abs_srcdir "/qemublocktestdata/imagecreate/";
+
+static virStorageSourcePtr
+testQemuImageCreateLoadDiskXML(const char *name,
+                               virDomainXMLOptionPtr xmlopt)
+
+{
+    virDomainSnapshotDiskDefPtr diskdef = NULL;
+    VIR_AUTOPTR(xmlDoc) doc = NULL;
+    VIR_AUTOPTR(xmlXPathContext) ctxt = NULL;
+    xmlNodePtr node;
+    VIR_AUTOFREE(char *) xmlpath = NULL;
+    virStorageSourcePtr ret = NULL;
+
+    if (virAsprintf(&xmlpath, "%s%s.xml",
+                    testQemuImageCreatePath, name) < 0)
+        return NULL;
+
+    if (!(doc = virXMLParseFileCtxt(xmlpath, &ctxt)))
+        return NULL;
+
+    if (!(node = virXPathNode("//disk", ctxt))) {
+        VIR_TEST_VERBOSE("failed to find <source> element\n");
+        return NULL;
+    }
+
+    if (VIR_ALLOC(diskdef) < 0)
+        return NULL;
+
+    if (virDomainSnapshotDiskDefParseXML(node, ctxt, diskdef,
+                                         VIR_DOMAIN_DEF_PARSE_STATUS,
+                                         xmlopt) == 0)
+        VIR_STEAL_PTR(ret, diskdef->src);
+
+    virDomainSnapshotDiskDefFree(diskdef);
+    return ret;
+}
+
+
+static int
+testQemuImageCreate(const void *opaque)
+{
+    struct testQemuImageCreateData *data = (void *) opaque;
+    VIR_AUTOPTR(virJSONValue) protocolprops = NULL;
+    VIR_AUTOPTR(virJSONValue) formatprops = NULL;
+    VIR_AUTOUNREF(virStorageSourcePtr) src = NULL;
+    VIR_AUTOCLEAN(virBuffer) debug = VIR_BUFFER_INITIALIZER;
+    VIR_AUTOCLEAN(virBuffer) actualbuf = VIR_BUFFER_INITIALIZER;
+    VIR_AUTOFREE(char *) jsonprotocol = NULL;
+    VIR_AUTOFREE(char *) jsonformat = NULL;
+    VIR_AUTOFREE(char *) actual = NULL;
+    VIR_AUTOFREE(char *) jsonpath = NULL;
+
+    if (!(src = testQemuImageCreateLoadDiskXML(data->name, data->driver->xmlopt)))
+        return -1;
+
+    if (data->backingname &&
+        !(src->backingStore = testQemuImageCreateLoadDiskXML(data->backingname,
+                                                             data->driver->xmlopt)))
+        return -1;
+
+    if (testQemuDiskXMLToJSONFakeSecrets(src) < 0)
+        return -1;
+
+    /* fake some sizes */
+    src->capacity = 1337;
+    src->physical = 42;
+
+    if (qemuDomainValidateStorageSource(src, data->qemuCaps) < 0)
+        return -1;
+
+    if (qemuBlockStorageSourceCreateGetStorageProps(src, &protocolprops) < 0)
+        return -1;
+
+    if (qemuBlockStorageSourceCreateGetFormatProps(src, src->backingStore, &formatprops) < 0)
+        return -1;
+
+    if (formatprops) {
+        if (!(jsonformat = virJSONValueToString(formatprops, true)))
+            return -1;
+
+        if (testQEMUSchemaValidate(formatprops, data->schemaroot, data->schema,
+                                   &debug) < 0) {
+            VIR_AUTOFREE(char *) debugmsg = virBufferContentAndReset(&debug);
+            VIR_TEST_VERBOSE("blockdev-create format json does not conform to QAPI schema");
+            VIR_TEST_DEBUG("json:\n%s\ndoes not match schema. Debug output:\n %s",
+                           jsonformat, NULLSTR(debugmsg));
+            return -1;
+        }
+        virBufferFreeAndReset(&debug);
+    }
+
+    if (protocolprops) {
+        if (!(jsonprotocol = virJSONValueToString(protocolprops, true)))
+            return -1;
+
+        if (testQEMUSchemaValidate(protocolprops, data->schemaroot, data->schema,
+                                   &debug) < 0) {
+            VIR_AUTOFREE(char *) debugmsg = virBufferContentAndReset(&debug);
+            VIR_TEST_VERBOSE("blockdev-create protocol json does not conform to QAPI schema");
+            VIR_TEST_DEBUG("json:\n%s\ndoes not match schema. Debug output:\n %s",
+                           jsonprotocol, NULLSTR(debugmsg));
+            return -1;
+        }
+        virBufferFreeAndReset(&debug);
+    }
+
+    virBufferStrcat(&actualbuf, "protocol:\n", NULLSTR(jsonprotocol),
+                    "\nformat:\n", NULLSTR(jsonformat), NULL);
+    virBufferTrim(&actualbuf, "\n", -1);
+    virBufferAddLit(&actualbuf, "\n");
+
+    if (virAsprintf(&jsonpath, "%s%s.json",
+                    testQemuImageCreatePath, data->name) < 0)
+        return -1;
+
+    if (!(actual = virBufferContentAndReset(&actualbuf)))
+        return -1;
+
+    return virTestCompareToFile(actual, jsonpath);
+}
+
+
+static int
+testQemuDiskXMLToPropsValidateFileSrcOnly(const void *opaque)
+{
+    struct testQemuDiskXMLToJSONData *data = (void *) opaque;
+    VIR_AUTOCLEAN(virBuffer) buf = VIR_BUFFER_INITIALIZER;
+    VIR_AUTOFREE(char *) jsonpath = NULL;
+    VIR_AUTOFREE(char *) actual = NULL;
+    size_t i;
+
+    if (data->fail)
+        return EXIT_AM_SKIP;
+
+    if (virAsprintf(&jsonpath, "%s%s-srconly.json",
+                    testQemuDiskXMLToJSONPath, data->name) < 0)
+        return -1;
+
+    for (i = 0; i < data->npropssrc; i++) {
+        VIR_AUTOFREE(char *) jsonstr = NULL;
+
+        if (!(jsonstr = virJSONValueToString(data->propssrc[i], true)))
+            return -1;
+
+        virBufferAdd(&buf, jsonstr, -1);
+    }
+
+    if (virBufferCheckError(&buf) < 0)
+        return -1;
+
+    actual = virBufferContentAndReset(&buf);
+
+    return virTestCompareToFile(actual, jsonpath);
+}
+
+
 static int
 mymain(void)
 {
@@ -320,6 +518,7 @@ mymain(void)
     virQEMUDriver driver;
     struct testBackingXMLjsonXMLdata xmljsonxmldata;
     struct testQemuDiskXMLToJSONData diskxmljsondata;
+    struct testQemuImageCreateData imagecreatedata;
     char *capslatest_x86_64 = NULL;
     virQEMUCapsPtr caps_x86_64 = NULL;
 
@@ -327,6 +526,7 @@ mymain(void)
         return EXIT_FAILURE;
 
     diskxmljsondata.driver = &driver;
+    imagecreatedata.driver = &driver;
 
     if (!(capslatest_x86_64 = testQemuGetLatestCapsForArch("x86_64", "xml")))
         return EXIT_FAILURE;
@@ -338,6 +538,7 @@ mymain(void)
         return EXIT_FAILURE;
 
     diskxmljsondata.qemuCaps = caps_x86_64;
+    imagecreatedata.qemuCaps = caps_x86_64;
 
     virTestCounterReset("qemu storage source xml->json->xml ");
 
@@ -411,6 +612,8 @@ mymain(void)
         diskxmljsondata.name = nme; \
         diskxmljsondata.props = NULL; \
         diskxmljsondata.nprops = 0; \
+        diskxmljsondata.propssrc = NULL; \
+        diskxmljsondata.npropssrc = 0; \
         diskxmljsondata.fail = fl; \
         if (virTestRun("disk xml to props " nme, testQemuDiskXMLToProps, \
                        &diskxmljsondata) < 0) \
@@ -421,6 +624,9 @@ mymain(void)
         if (virTestRun("disk xml to props validate file " nme, \
                        testQemuDiskXMLToPropsValidateFile,  &diskxmljsondata) < 0) \
             ret = -1; \
+        if (virTestRun("disk xml to props source only validate file " nme, \
+                       testQemuDiskXMLToPropsValidateFileSrcOnly,  &diskxmljsondata) < 0) \
+            ret = -1; \
         testQemuDiskXMLToPropsClear(&diskxmljsondata); \
     } while (0)
 
@@ -480,6 +686,41 @@ mymain(void)
     TEST_DISK_TO_JSON("block-raw-noopts");
     TEST_DISK_TO_JSON("block-raw-reservations");
 
+# define TEST_IMAGE_CREATE(testname, testbacking) \
+    do { \
+        imagecreatedata.name = testname; \
+        imagecreatedata.backingname = testbacking; \
+        if (virTestRun("image create xml to props " testname, testQemuImageCreate, \
+                       &imagecreatedata) < 0) \
+            ret = -1; \
+    } while (0)
+    imagecreatedata.schema = diskxmljsondata.schema;
+    if (virQEMUQAPISchemaPathGet("blockdev-create/arg-type/options",
+                                 imagecreatedata.schema,
+                                 &imagecreatedata.schemaroot) < 0 ||
+        !imagecreatedata.schemaroot) {
+        VIR_TEST_VERBOSE("failed to find schema entry for blockdev-create\n");
+        ret = -1;
+        goto cleanup;
+    }
+
+    TEST_IMAGE_CREATE("raw", NULL);
+    TEST_IMAGE_CREATE("raw-nbd", NULL);
+    TEST_IMAGE_CREATE("luks-noopts", NULL);
+    TEST_IMAGE_CREATE("luks-encopts", NULL);
+    TEST_IMAGE_CREATE("qcow2", NULL);
+    TEST_IMAGE_CREATE("qcow2-luks-noopts", NULL);
+    TEST_IMAGE_CREATE("qcow2-luks-encopts", NULL);
+    TEST_IMAGE_CREATE("qcow2-backing-raw", "raw");
+    TEST_IMAGE_CREATE("qcow2-backing-raw-nbd", "raw-nbd");
+    TEST_IMAGE_CREATE("qcow2-backing-luks", "luks-noopts");
+    TEST_IMAGE_CREATE("qcow2-luks-encopts-backing", "qcow2");
+
+    TEST_IMAGE_CREATE("network-gluster-qcow2", NULL);
+    TEST_IMAGE_CREATE("network-rbd-qcow2", NULL);
+    TEST_IMAGE_CREATE("network-ssh-qcow2", NULL);
+    TEST_IMAGE_CREATE("network-sheepdog-qcow2", NULL);
+
  cleanup:
     virHashFree(diskxmljsondata.schema);
     qemuTestDriverFree(&driver);
