@@ -72,18 +72,18 @@ VIR_MOCK_WRAP_RET_ARGS(dbus_connection_send_with_reply_and_block,
         int is_authorized = 1;
         int is_challenge = 0;
 
-        if (virDBusMessageRead(message,
-                               "(sa{sv})sa&{ss}us",
-                               &type,
-                               3,
-                               &pidkey, "u", &pidval,
-                               &timekey, "t", &timeval,
-                               &uidkey, "i", &uidval,
-                               &actionid,
-                               &detailslen,
-                               &details,
-                               &allowInteraction,
-                               &cancellationId) < 0)
+        if (virDBusMessageDecode(message,
+                                 "(sa{sv})sa&{ss}us",
+                                 &type,
+                                 3,
+                                 &pidkey, "u", &pidval,
+                                 &timekey, "t", &timeval,
+                                 &uidkey, "i", &uidval,
+                                 &actionid,
+                                 &detailslen,
+                                 &details,
+                                 &allowInteraction,
+                                 &cancellationId) < 0)
             goto error;
 
         if (STREQ(actionid, "org.libvirt.test.success")) {
