@@ -51,67 +51,70 @@ static int testURIParse(const void *args)
     const struct URIParseData *data = args;
     char *uristr = NULL;
     size_t i;
+    bool fail = false;
 
     if (!(uri = virURIParse(data->uri)))
         goto cleanup;
 
     if (STRNEQ(uri->scheme, data->scheme)) {
-        VIR_DEBUG("Expected scheme '%s', actual '%s'",
-                  data->scheme, uri->scheme);
-        goto cleanup;
+        VIR_TEST_DEBUG("Expected scheme '%s', actual '%s'",
+                       data->scheme, uri->scheme);
+        fail = true;
     }
 
     if (STRNEQ(uri->server, data->server)) {
-        VIR_DEBUG("Expected server '%s', actual '%s'",
-                  data->server, uri->server);
-        goto cleanup;
+        VIR_TEST_DEBUG("Expected server '%s', actual '%s'",
+                       data->server, uri->server);
+        fail = true;
     }
 
     if (uri->port != data->port) {
-        VIR_DEBUG("Expected port '%d', actual '%d'",
-                  data->port, uri->port);
-        goto cleanup;
+        VIR_TEST_DEBUG("Expected port '%d', actual '%d'",
+                       data->port, uri->port);
+        fail = true;
     }
 
     if (STRNEQ_NULLABLE(uri->path, data->path)) {
-        VIR_DEBUG("Expected path '%s', actual '%s'",
-                  data->path, uri->path);
-        goto cleanup;
+        VIR_TEST_DEBUG("Expected path '%s', actual '%s'",
+                       data->path, uri->path);
+        fail = true;
     }
 
     if (STRNEQ_NULLABLE(uri->query, data->query)) {
-        VIR_DEBUG("Expected query '%s', actual '%s'",
-                  data->query, uri->query);
-        goto cleanup;
+        VIR_TEST_DEBUG("Expected query '%s', actual '%s'",
+                       data->query, uri->query);
+        fail = true;
     }
 
     if (STRNEQ_NULLABLE(uri->fragment, data->fragment)) {
-        VIR_DEBUG("Expected fragment '%s', actual '%s'",
-                  data->fragment, uri->fragment);
-        goto cleanup;
+        VIR_TEST_DEBUG("Expected fragment '%s', actual '%s'",
+                       data->fragment, uri->fragment);
+        fail = true;
     }
 
     for (i = 0; data->params && data->params[i].name && i < uri->paramsCount; i++) {
         if (STRNEQ_NULLABLE(data->params[i].name, uri->params[i].name)) {
-            VIR_DEBUG("Expected param name %zu '%s', actual '%s'",
-                      i, data->params[i].name, uri->params[i].name);
-            goto cleanup;
+            VIR_TEST_DEBUG("Expected param name %zu '%s', actual '%s'",
+                           i, data->params[i].name, uri->params[i].name);
+            fail = true;
         }
         if (STRNEQ_NULLABLE(data->params[i].value, uri->params[i].value)) {
-            VIR_DEBUG("Expected param value %zu '%s', actual '%s'",
-                      i, data->params[i].value, uri->params[i].value);
-            goto cleanup;
+            VIR_TEST_DEBUG("Expected param value %zu '%s', actual '%s'",
+                           i, data->params[i].value, uri->params[i].value);
+            fail = true;
         }
     }
     if (data->params && data->params[i].name) {
-        VIR_DEBUG("Missing parameter %zu %s=%s",
-                  i, data->params[i].name, data->params[i].value);
-        goto cleanup;
+        VIR_TEST_DEBUG("Missing parameter %zu %s=%s",
+                       i, data->params[i].name, data->params[i].value);
+        fail = true;
     }
     if (i != uri->paramsCount) {
-        VIR_DEBUG("Unexpected parameter %zu %s=%s",
-                  i, uri->params[i].name, uri->params[i].value);
-        goto cleanup;
+        for (; i < uri->paramsCount; i++) {
+            VIR_TEST_DEBUG("Unexpected parameter %zu %s=%s",
+                           i, uri->params[i].name, uri->params[i].value);
+        }
+        fail = true;
     }
 
     VIR_FREE(uri->query);
@@ -121,11 +124,14 @@ static int testURIParse(const void *args)
         goto cleanup;
 
     if (STRNEQ(uristr, data->uri_out)) {
-        VIR_DEBUG("URI did not roundtrip, expect '%s', actual '%s'",
-                  data->uri_out, uristr);
-        goto cleanup;
+        VIR_TEST_DEBUG("URI did not roundtrip, expect '%s', actual '%s'",
+                       data->uri_out, uristr);
+        fail = true;
     }
 
+    if (fail)
+        goto cleanup;
+
     ret = 0;
  cleanup:
     VIR_FREE(uristr);
@@ -172,6 +178,14 @@ mymain(void)
     TEST_PARSE("test://[2001:41c8:1:4fd4::2]:123/system", "test", "2001:41c8:1:4fd4::2", 123, "/system", NULL, NULL, NULL, NULL);
     TEST_PARSE("gluster+rdma://example.com:1234/gv0/vol.img", "gluster+rdma", "example.com", 1234, "/gv0/vol.img", NULL, NULL, NULL, NULL);
 
+    virURIParam spiceparams[] = {
+        { (char *) "tlsSubject", (char *) "C=XX,L=Testtown,O=Test Company,CN=tester.test", false },
+        { NULL, NULL, false },
+    };
+    TEST_FULL("spice://[3ffe::104]:5900/?tlsSubject=C=XX,L=Testtown,O=Test%20Company,CN=tester.test",
+              "spice://[3ffe::104]:5900/?tlsSubject=C%3dXX%2cL%3dTesttown%2cO%3dTest%20Company%2cCN%3dtester%2etest",
+              "spice", "3ffe::104", 5900, "/", "tlsSubject=C=XX,L=Testtown,O=Test%20Company,CN=tester.test", NULL, NULL, spiceparams);
+
     virURIParam params1[] = {
         { (char*)"foo", (char*)"one", false },
         { (char*)"bar", (char*)"two", false },
