@@ -40,16 +40,14 @@
 #include "virerror.h"
 #include "virbuffer.h"
 #include "viralloc.h"
-#include <libvirt/libvirt-qemu.h>
-#include <libvirt/libvirt-lxc.h>
 #include "virfile.h"
 #include "virthread.h"
 #include "vircommand.h"
-#include "conf/domain_conf.h"
 #include "virtypedparam.h"
 #include "virstring.h"
 #include "virgettext.h"
 
+#include "virsh-checkpoint.h"
 #include "virsh-console.h"
 #include "virsh-domain.h"
 #include "virsh-domain-monitor.h"
@@ -832,6 +830,7 @@ static const vshCmdGrp cmdGroups[] = {
     {VIRSH_CMD_GRP_DOM_MANAGEMENT, "domain", domManagementCmds},
     {VIRSH_CMD_GRP_DOM_MONITORING, "monitor", domMonitoringCmds},
     {VIRSH_CMD_GRP_HOST_AND_HV, "host", hostAndHypervisorCmds},
+    {VIRSH_CMD_GRP_CHECKPOINT, "checkpoint", checkpointCmds},
     {VIRSH_CMD_GRP_IFACE, "interface", ifaceCmds},
     {VIRSH_CMD_GRP_NWFILTER, "filter", nwfilterCmds},
     {VIRSH_CMD_GRP_NETWORK, "network", networkCmds},
